#include "SandBox.h"

#define MED 0
#define CRS 1

double SandBox::K_s[2] = { 3.26, 4.05 };
double SandBox::theta_s[2] = { 0.44, 0.446 };
double SandBox::theta_r[2] = { 0.067, 0.047 };
double SandBox::alpha[2] = { 9.13, 14.6 };
double SandBox::n[2] = { 4.27, 10.16 };

double SandBox::width() const
{
	return 1.5;
}

double SandBox::height() const
{
	return 0.16;
}

double SandBox::h0(double x, double y) const
{
	return -0.1;
}

double SandBox::qs(double x, double y) const
{
	if (y > 0.13 && x == 0.0)
	{
		return 1.62;
	}
	else
	{
		return 0.0;
	}
}

int SandBox::get_medium_index(double x, double y) const
{
	if (y >= 0.12) return MED;
	else return CRS;
}

double SandBox::K(double x, double y, double h) const
{
	int i = get_medium_index(x,y);
	double m = 1.0-1.0/n[i];
	if (!saturated(x,y,h))
	{
		return K_s[i] * 
		pow((1.0-pow(fabs(alpha[i]*h),n[i]-1.0))*
		pow(fabs(1.0+pow(fabs(alpha[i]*h),n[i])),-m),2.0) /
		pow(fabs(1.0 + pow(fabs(alpha[i]*h),n[i])),m/2.0);
	}
	else
	{
		return K_s[i];
	}
}

double SandBox::theta(double x, double y, double h) const
{
	int i = get_medium_index(x,y);
	double m = 1.0-1.0/n[i];
	if (!saturated(x,y,h))
	{
		return (theta_s[i]-theta_r[i])*
		pow(fabs(1.0+pow(fabs(alpha[i]*h),n[i])),-m)+theta_r[i];
	}
	else
	{
		return theta_s[i];
	}
}

double SandBox::C_Ss(double x, double y, double h) const
{
	int i = get_medium_index(x,y);
	double m = 1.0-1.0/n[i];
	if (!saturated(x,y,h))
	{
		return alpha[i]*m*n[i]*(theta_s[i]-theta_r[i]) *
		pow(fabs(alpha[i]*h),n[i]-1.0) / 
		pow(1.0+pow(fabs(alpha[i]*h),n[i]),m+1.0);
	}
	else
	{
		return alpha[i]+n[i];
	}
}

bool SandBox::saturated(double x, double y, double h) const
{
	return h >= -ADEVS_EPSILON;
}
