#ifndef __sandbox_h_
#define __sandbox_h_
#include "cell.h"

/**
Material parameters for the sand box described in "A Numerical Model
for Water Flow and Chemical Transport in Variably Saturated Porous
Media" by Jim Yeh, et. al.
*/
class SandBox: public MaterialParams
{
	public:
		double width() const;
		double height() const;
		/// Get the initial pressure
		double h0(double x, double y) const;
		/// Get the pressure flux
		double qs(double x, double y) const;
		/// Compute hydraulic conductivity
		double K(double x, double y, double h) const;
		/// Compute moisture capacitance
		double C_Ss(double x, double y, double h) const;
		/// Compute moisture content
		double theta(double x, double y, double h) const;
		bool saturated(double x, double y, double h) const;
	private:
		/// Material properties
		static double K_s[2];
		static double theta_s[2];
		static double theta_r[2];
		static double alpha[2];
		static double n[2];

		int get_medium_index(double x, double y) const;
};

#endif
