C****************************************************************************
C                                                                           *
C                     xUNSAT1D.FOR                                          *
C                                                                           *
C  UNSAT1D.FOR solves one-dimensional Richard's equation                    *
C  with either prescribed heads or flux boundary conditions,                *
C  using the Galerkin finite element method.                                *
C  The finite element formulation can be found in                           *
C  KHALEEL and YEH (Feb. 1985, GROUND WATER).  However, the                 *
C  original program has been modified extensively.  The modifications       *
C  are:                                                                     *
C    1) Automatic time stepping scheme is implimented.                      *
C    2) This program can simulate steady or transient VERTICAL              *
C       OR HORIZONTAL, and variably saturated flow in homogeneous           *
C       or heterogeneous soils.                                             *
C               Jim Yeh, May. 5, 1990.                                      *
C                                                                           *
C    Spring 90:                                                             *
C     UNSATURATED SOIL PARAMETERS MAY BE REPRESENTED BY THE FOLLOWING       *
C     MODELS:                                                               *
C               IDM(I) = 1: VAN GENUCHTEN MODEL                             *
C                      = 2: DEXPONENTIAL MODEL                              *
C                      = 3: GARDNER-RUSSO MODEL                             *
C                      = 4: EXAMPLE OF USER-SPECIFIED MODEL                 *
C                                                                           *
C                                                                           *
C               Amado Guzman-Guzman, May 23, 1990                           *
C                                                                    
C       update:  Sept. 23, 1991 by Jim Yeh
C
C       January 15, 1993.  Jim Yeh
C       Newton-Raphson scheme was implemented back to picard.for
C       and the program is called UNSAT1D.FOR
C       
C       xunsat1d.for is modified version of unsat1d.for.
c       In this code, a mixed form of Richard's equation is used
c       to preserve mass.
C
C******************************************************************************
      PROGRAM XUNSAT1D
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      CHARACTER    III*4
      CHARACTER*64 TITLE(2),SCHM(2),FLOW(2)
      CHARACTER*4  PROJECT(20)
      CHARACTER*14 MDL(10)
      INTEGER*4    HH,MM,SEC,HD
      REAL*8       L
      DIMENSION PTPSI(501),PSIH(501),NBC(2),BC(2),Q(2),PIPSI(501),
     *SF(2,2,501),RHS(2,2,501),F(2,501),g(2,501),
     *TPSI(501),KODE(501),TIMOUT(20)
      COMMON A(501),B(501),C(501),D(501),PSI(501)
      COMMON/BM/DELT,DELZ,EPS,NODES
      COMMON/SL/XKS(10),ALFA(10),XN(10),WCS(10),WCR(10),SS(10),IDM(10)
      DATA TITLE/' STEADY STATE SIMULATION ',' TRANSIENT SIMULATION'/
      DATA SCHM/'Picard','Newton-Raphson'/
      DATA FLOW/'Horizontal Flow',' Vertical Flow'/
      DATA MDL/'van Genuchten ',' Exponential ','Gardner-Russo','  User-
     *sup.  ',6*'            '/

      CQIN  = 0.0
      CQOUT = 0.0
      CQNET = 0.0
      CCIWC = 0.0
      CERRWC= 0.0
      BETA = 0.0
      IP = 1

C.....Open input files
c       open(99,file='test.out',status='unknown')
      OPEN (UNIT=7, FILE='unsat1d.in',STATUS='UNKNOWN')
                                         
C.....Open output files
C         unsat1d.out is the output with all information
C         unsat1d.dat is the output file for tecplot
C         restar.dat  is the data file for restart
c
      OPEN (UNIT=11,FILE='unsat1d.out',STATUS='UNKNOWN')
      OPEN (UNIT=20,FILE='unsat1d.dat',STATUS='UNKNOWN')
      OPEN (UNIT=30,FILE='restar.dat' ,STATUS='UNKNOWN')


      WRITE (*,*)' Interactive (1) or data file input (2): ?'
      READ(*,*)ISF

      IF(ISF.EQ.1)THEN
         WRITE(*,*)
     *   'Select flow regime:(1) STEADY STATE, (2) TRANSIENT:'
         READ (*,*)ISTATE
         IF(ISTATE.NE.1) THEN
                WRITE(*,*)' Specify no. of output times: '
                READ(*,*)IPRDT
                WRITE(*,*)' Specify output times: '
                READ(*,*)(TIMOUT(I),I=1,IPRDT)
           ELSE
                WRITE(*,*)' Input pseudotransient weighting factor:'
                READ(*,*)BETA
         ENDIF
         WRITE(*,*)' Detailed iter. information printout (y/n)? '
         READ(*,'(A)')III
         IF(III.EQ.'Y'.OR.III.EQ.'y')THEN
                IIP=1
         ELSE
                IIP=0
         ENDIF


      ELSE
         OPEN (UNIT=8,FILE='CONTROL',STATUS='UNKNOWN')
         READ(8,*)ISTATE
         IF(ISTATE.EQ.2)THEN
            READ(8,*)IPRDT
            READ(8,*)(TIMOUT(I),I=1,IPRDT)
         ELSE
            READ(8,*)BETA
         ENDIF
         READ(8,*)IIP
      ENDIF
c         
c....... added for newton option
c
         write(*,*)'Choice of iteration scheme: 1-Picard, 2-Newton'
         read (*,*) newton
         write(*,*) 'Use guess solution? 1-yes, 2-no.'
         read (*,*) iguess
c
c
c........get the system starting time
c
      CALL GETTIM(HH,MM,SEC,HD)
      TIME0 =3600.*HH+60.*MM+SEC+0.01*HD
      WRITE (11,1) TITLE(ISTATE)
    1 FORMAT(/,20X,' UNSAT1D.FOR (written by Jim Yeh, January 1993)',//,
     $20X,10('='),A25,10('=')/)

C....Read input dat from the input data file
      
      READ (7, '(20A4)') (PROJECT(I),I=1,20)
      WRITE(11,'(A8,2X,20A4)') 'PROJECT:',(PROJECT(I),I=1,20)
C....................................................................
C
C     SOIL PROPERTY LOOP
C
C....................................................................
      READ (7,*)NOS
      WRITE(11,110)
 110  FORMAT(/,11X,'*** Parameters of Soil Properties Models***'//,
     $1X,'SOIL TYPE ',
     $'    KSAT      ','     ALPHA    ',
     $'      N       ','     WCS      ',
     $'    WCR      ','    Ss    ','    MODEL   ',/)

C.....Read soil properties.

      DO 120 J = 1,NOS

C.....Read input data from the input data file

c      READ (7,'(A)')NAME
      READ (7,*)XKS(J),ALFA(J),XN(J),WCS(J),WCR(J),SS(J),IDM(J)
      WRITE (11,115)J,XKS(J),ALFA(J),XN(J),WCS(J),WCR(J),SS(J),
     *              MDL(IDM(J))
 115  FORMAT(' ',I3,1X,6E13.5,2X,A13)
 120  CONTINUE

C.....Read simulation parameters

      WRITE(11,125)
 125  FORMAT(//,10X,10('-'),' SIMULATION PARAMETERS ',10('-'))
      READ (7,*)NODES,DELZ,DELT,DTMIN,EPS,XMULT,DELMAX,
     *TOL,MAXIT,INTEG
      WRITE(11,126)NODES,DELZ,DELT,DTMIN,EPS,XMULT,DELMAX,
c     
c-modified for newton
c
     *TOL,MAXIT,FLOW(INTEG),SCHM(Newton)
  126 FORMAT(' ',/,
     *11X,'TOTAL NO. OF NODES......................= ',I13,/,
     *11X,'DELZ....................................= ',F13.3,/,
     *11X,'DELT....................................= ',E13.5,/,
     *11X,'MINIMUM DELT............................= ',E13.5,/,
     *11X,'TIME WEIGHTING FACTOR, EPS..............= ',F13.3,/,
     *11X,'TIME STEP MULTIPLIER, XMULT.............= ',F13.3,/,
     *11X,'MAXIMUM DELT, DELMAX....................= ',E13.5,/,
     *11X,'TOLERANCE, TOL..........................= ',E13.5,/,
     *11X,'MAXIMUM NO. OF ITERATIONS, MAXIT........= ',I13,/,
     *11X,'FLOW....................................= ',A64,/,
     *11X,'ITERATION SCHEME........................= ',A64)

      NODE1=NODES-1
      NELEM=NODE1
      L=DELZ
      NELEM1=NELEM-1
      DELT1=DELT

C.....Read soil type of each element

      READ (7,*) (KODE(I),I=1,NELEM)
      WRITE(11,127)
 127  FORMAT(/11X,20('-'),' ELEMENT SOIL TYPE INFORMATION ',20('-'),
     #/,10('ELEM SOIL'))
      WRITE(11,128) (I,KODE(I),I=1,NELEM)
 128  FORMAT(10(I3,2X,I3,1X))
C
C====================================================================
C
C      PSI   -- Pressure values at current iteration and time level.
C      PTPSI -- Pressure values at previous time level.
C      PIPSI -- Pressure values at previous iteration level.
C      TPSI  -- Pressure values at second previous time level.
C      PSIH  -- Pressure values at 1/2 time level if picard iteration
C               is chosen.
C
C====================================================================

C.....Read initial condition for transient flow or
C     initial guess values for steady-state flow
C     STRTIM is the starting time
C
      IF(PROJECT(1).EQ.'REST') THEN
        OPEN (UNIT=10, FILE='RESTAR.DAT',STATUS='UNKNOWN')
        READ (10,*) STRTIM
        READ (10,'(5E13.5)') CQIN,CQOUT,CQNET,CCIWC,CERRWC
        READ (10,'(2E13.5)') (PTPSI(I),TPSI(I),I=1,NODES)
      ELSE
        OPEN (UNIT=10, FILE='ic.in',STATUS='UNKNOWN')
        READ (10,*) STRTIM
        READ (10,*) (PTPSI(I),I=1,NODES)
      ENDIF

      WRITE(11,130)
 130  FORMAT (' ',/,10X,10('-'),' INITIAL CONDITION ',10('-'),/)
      WRITE (11,132) (PTPSI(I),I=1,NODES)
 132  FORMAT (10F8.2)

C.....Initialize the arrays

      DO 140 I=1,NODES
      PSI(I) = PTPSI(I)
      IF(PROJECT(1).EQ.'REST') TPSI (I) = PSI(I)
      PIPSI(I) = PTPSI(I)
      PSIH(I)  = PTPSI(I)
  140 CONTINUE

C.....Boundary conditions

      WRITE(11,142)
 142  FORMAT(//11X,'------BOUNDARY CONDITIONS ------',
     */,11X,'NODE',1X,'TYPE',5X,'PSI',8X,'FLUX')

      DO 145 I=1,2
      N=1
      IF(I.EQ.2)N=NODES

C.....NBC = 1  Flux boundary condition  Q positive downward
C.....NBC = 0  Head boundary condition

      READ   (7,*) NBC(I),BC(I),Q(I)
      WRITE  (11,144)N,NBC(I),BC(I),Q(I)
 144  FORMAT (' ',10X,2I5,F10.2,E13.5)

      IF(NBC(I).EQ.0) THEN
        PSI(N) = BC(I)
        PIPSI(N) = BC(I)
        PTPSI(N) = BC(I)
        TPSI(N)  = BC(I)
        PSIH(N)  = BC(I)
      ENDIF
145   CONTINUE

      TIME=STRTIM
      LA=0
      IP=1
      DTT=0.0

C....JFLAG an indicator for nonconvergence in transient simulation
C    NFLAG an indicator for DELT adjustment to fit printout time
C    NFLAG = 1, the adjustment was done at last time step.

      JFLAG = 0
      NFLAG = 0
C====================================================================
C
C              SIMULATION STARTS HERE
C
C                   TIME LOOP
C
C====================================================================
98    CONTINUE
      LA=LA+1

C.....ITIMR is the counter for number of delt reductions

      ITIMR = 0

C......New delt loop

99    CONTINUE

C......Predict psi at t+1/2 time step by linear extrapolation.

      IF(LA.GT.1)DTT=DELT/DELT1
      if(newton.ne.2) then
          DO 100 I=1,NODES
          PSIH(I)=PTPSI(I)+(1.-EPS)*DTT*(PTPSI(I)-TPSI(I))
100       CONTINUE
      endif
C
C.... if steady state flow is used,
c     provide initial guess values
c
           IF(Iguess.EQ.1) THEN
           open(40,file='guess.dat')       
           do i =1,nodes
           read(40,*) z, PSIH(I)
           pipsi(i) = psih(i)
           enddo
           endif           

      EPSLON1=1.0E+28
      EPSLON2=1.0E+28
      EPSLON =1.0E+28
      ITER=0

C====================================================================
C
C             ITERATION LOOP
C
C====================================================================
900   CONTINUE

      ITER = ITER + 1

C......Set up element stiffness matrix
C
C...If picard iteration is used, psih(k) is the pressure head
C   at t + 1/2 .
C   If netwon iteration is used, psih(k) is the pressure head
C   at the previous iteration and pipsi(k) it the pressure
C   at the iteration before psih(k).
c

      DO 200 K=1,NELEM
      M=KODE(K)
      P1=PSIH(K)
      P2=PSIH(K+1)
c      IF(P1.GT.0.0) P1 = 0.0
c      IF(P2.GT.0.0) P2 = 0.0

C.....Determine hydraulic conductivity and moisture capacity
C     if p1 or p2 ge or gt zero, K=ksat and cm = Ss
C
      XK1  = XK(P1,M)
      XK2  = XK(P2,M)
      XKAV = 0.5*(XK1 + XK2)

      C1   = CM(P1,M)
      C2   = CM(P2,M)

C......Determine individual elements in each element matrix

      A11=XKAV/L
      A12=-A11
      A21=A12
      A22=A11

      b11 =0.0
      B12=0.0
      B21=0.0
      b22 = 0.0
      if(newton.eq.1) then
      B11=0.5*c1
      B22=0.5*c2
      endif

      TL = 0.0
      IF(ISTATE.NE.1) TL = L/DELT
      IF(ISTATE.EQ.1) EPS = 1.0
      SF(1,1,K)=EPS*A11+B11*TL
      SF(1,2,K)=EPS*A12
      SF(2,1,K)=EPS*A21
      SF(2,2,K)=EPS*A22+B22*TL

C.....Construct the right-hand side
      If(newton.eq.1) then
      RHS(1,1,K)=(EPS-1.)*A11+B11*TL
      RHS(1,2,K)=(EPS-1.)*A12
      RHS(2,1,K)=(EPS-1.)*A21
      RHS(2,2,K)=(EPS-1.)*A22+B22*TL
      endif

C.....Compute the gravity term which is the averaged K.
C     This will be used in mass balance calculation later.

      F(1,K)=-XKAV
      F(2,K)= XKAV
      
      IF(NEWTON.NE.2) GO TO 200
C**********************************************************
C
C  NEWTON-RAPHSON'S ITERATION SCHEME
C  COMPUTE G MATRIX (RIGHT-HAND SIDE) FOR EACH ELEMENT
C
C**********************************************************

      G(1,K)=A11*P1+A12*P2
      G(2,K)=A21*P1+A22*P2
      
      IF(ISTATE.EQ.2) THEN
      pp1=ptpsi(k)
	pp2=ptpsi(k+1)

	wce=wcs(m)-wcr(m)
C
C.....Moisture content at previous time
C
      IF(pp1.LT.0.0) THEN
        PWC1 = SE(pp1,M)*WCE+WCR(M)
      ELSE
        PWC1 = SS(M)*PP1 + WCS(M)
      ENDIF

      IF(PP2.LT.0.0) THEN
        PWC2 = SE(PP2,M)*WCE+WCR(M)
      ELSE
        PWC2 = SS(M)*PP2 + WCS(M)
      ENDIF

      
C
C.....Moisture content at current time
C
      IF(P1.LT.0.0) THEN
        WC1 = SE(P1,M)*WCE+WCR(M)
      ELSE
        WC1 = SS(M)*P1 + WCS(M)
      ENDIF
      IF(P2.LT.0.0) THEN
        WC2 = SE(P2,M)*WCE+WCR(M)
      ELSE
        WC2 = SS(M)*P2 + WCS(M)
      ENDIF

      
C.....Change in moisture content at node1 and node2 the element during delt

      DWC1 = WC1 - PWC1
	dwc2 = wc2 - pwc2

C----  Adding the change in moisture content  (mixed form) 

        G(1,K)=G(1,K)+TL*dwc1*0.5
        G(2,K)=G(2,K)+TL*dwc2*0.5


      ENDIF
C........add gravity term

      IF(INTEG.EQ.2) THEN
        G(1,K)=G(1,K)-F(1,K)
        G(2,K)=G(2,K)-F(2,K)
      ENDIF

C
C  FORMATION OF THE JACOBIAN OF EACH ELEMENT MATRIX
C
      PT1 = PiPSI(K)
      PT2 = PiPSI(K+1)
      IF (PiPSI(K).GE.0.) PT1 = 0.
      IF (PiPSI(K+1).GE.0.) PT2 = 0.
      IF (P1.GE.0.) P1 = 0.
      IF (P2.GE.0.) P2 = 0.

      DK1 = DK(P1,PT1,M)
      DK2 = DK(P2,PT2,M)
      DC1 = DC(P1,PT1,M)
      DC2 = DC(P2,PT2,M)

      DF1 = 0.
      DF2 = 0.
      IF(INTEG.NE.1) THEN
         DF1 = -0.5*DK1
         DF2 = +0.5*DK2
      ENDIF

      DELP=PSIH(K) - PSIH(K+1)

C
C   If ISTATE = 1: STEADY-STATE simulation, use pseudotransient
C   approach, by adding beta term to the diagonal element of the
C   Jacobian matrix.
C
      SF(1,1,K)=SF(1,1,K)+DELP*DK1*0.5/L-DF1 + BETA
      SF(1,2,K)=SF(1,2,K)+DELP*DK2*0.5/L+DF2
      SF(2,1,K)=SF(2,1,K)-DELP*DK1*0.5/L+DF1
      SF(2,2,K)=SF(2,2,K)-DELP*DK2*0.5/L-DF2 + BETA
C
C   Add transient term in the Jacobian Matrix	(mixed form formulation)
C
      IF(ISTATE.EQ.2) THEN
c      IF (P1.GE.0.) c1 = 0. Add nonzero term will stablize the solution.
c      IF (P2.GE.0.) c2 = 0. when p1 and p2 > 0 C1=C2=Ss a nonzero. Otherwise convergence problem.
	  SF(1,1,K)=SF(1,1,K)+TL*C1*0.5
        SF(1,2,K)=SF(1,2,K)
        SF(2,1,K)=SF(2,1,K)
        SF(2,2,K)=SF(2,2,K)+TL*C2*0.5
      ENDIF
C
C ***  END OF NEWTON RAPHSON MATRIX FORMULATION **
C
C

c....End of element loop
c
200   CONTINUE

C====================================================================
C
C       ASSEMBLE ELEMENT STIFFNES MATRICES INTO GLOBAL MATRIX
C
C====================================================================

C.....Top boundary condition

      IF(NBC(1).EQ.0) GOTO 210

C.....Constant flux boundary condition

      A(1)=0.0
      B(1)=SF(1,1,1)
      C(1)=SF(1,2,1)
      D(1)=RHS(1,1,1)*PTPSI(1)+RHS(1,2,1)*PTPSI(2)+Q(1)

C.....Adds gravity terms

      IF(INTEG.EQ.2)  D(1)=D(1)+F(1,1)
      if(newton.eq.2) d(1)=-g(1,1)+q(1)
      GOTO 220

C.....Constant head boundary condition

 210  A(1)=0.0
      B(1)=1.0
      C(1)=0.0
      D(1)=BC(1)
      if(newton.eq.2) d(1)=0.0
 220  CONTINUE

C.....Interior nodes

      DO 250 K=1,NELEM1
      K1=K+1
      A(K1)=SF(2,1,K)
      B(K1)=SF(2,2,K)+SF(1,1,K1)
      C(K1)=SF(1,2,K1)

      if(newton.eq.1) then
          AA=RHS(2,1,K)
          BB=RHS(2,2,K)+RHS(1,1,K1)
          CC=RHS(1,2,K1)
          D(K1)=AA*PTPSI(K)+BB*PTPSI(K+1)+CC*PTPSI(K+2)
C..................Adds gravity terms
          IF(INTEG.EQ.2) D(K1) = D(K1)+F(2,K)+F(1,K1)
      else
          d(k1)=-(g(2,k)+g(1,k1))
      endif
250   CONTINUE

C.....Lower boundary

      IF(NBC(2).EQ.0) GOTO 260

C.....Constant flux boundary condition

      A(NODES)=SF(2,1,NELEM)
      B(NODES)=SF(2,2,NELEM)
      C(NODES) = 0.0
      D(NODES)=RHS(2,1,NELEM)*PTPSI(NODES-1)+RHS(2,2,NELEM)
     1            *PTPSI(NODES)-Q(2)

C.....Adds gravity terms

      IF(INTEG.EQ.2) D(NODES) = D(NODES) + F(2,NELEM)
      if(newton.eq.2) d(nodes) = (-g(2,nelem)-q(2))
      GOTO 270

C....Constant head boundary condition

 260   A(NODES)=0.0
       B(NODES)=1.0
       C(NODES)=0.0
       D(NODES)=BC(2)
       if(newton.eq.2) d(nodes) = 0.0

 270   CONTINUE

C           write(99,991) iter,b(1),c(1),d(1),a(1)
C           write(99,991) iter,a(2),b(2),c(2),d(2) 
C 991       format(i4,4e13.5)
           
       CALL TRIDIA(NODES)

C.....Test of convergence

      EPSLON=0.0
      DO 300 I=1,NODES
         if(newton.eq.1) then
            CHG=DABS(PSI(I) - PIPSI(I))
         else
            chg=dabs(psi(i))
         endif
      IF(CHG.LT.EPSLON) GOTO 300
      EPSLON=CHG
      NMAX=I
 300  CONTINUE

C.....If the solution converges, go to print
      IF(EPSLON.LE.TOL.AND.ITER.NE.1) GOTO 500

C.....If the solution does not converge,

      IF(ISTATE.EQ.2) GO TO 501
C====================================================================
C
C              STEADY-STATE
C
C====================================================================
      IF(ITER.LE.MAXIT) GO TO 340
      WRITE(11,1015) MAXIT
      WRITE(* ,1015) MAXIT
1015  FORMAT(' ',/,'****  MAX. NO. OF ITERATIONS ',I4,
     $' IS EXCEEDED ',/,'      IN THIS STEADY-STATE SIMULATION ',/)
      JFLAG = 0
      GO TO 710

501   IF(ITER.LE.MAXIT) GO TO 340

C====================================================================
C
C              TRANSIENT FLOW
C    If the solution does not converge to the specified tolerance
C    and if MAXIT is exceeded and the previous head change
C    is less than the current one, reduce the time step and
C    restart this time step (as long as ITIMR<30).
C
C====================================================================

C.....If solution diverges with the previous DELT, go back
C     to label 99 and try with the reduced DELT

      DELTN = DELT
      DELT = DELT/10.
      IF(DELT.LT.DTMIN)THEN
        WRITE(*,'(/a)')
     *   ' ** SOLUTION DOES NOT CONVERGE: DELT<DTMIN **'
        WRITE(*,'(E13.5,I5)') DELT,ITIMR
        JFLAG = 1
        GO TO 710
      
      ELSE
C.......DELT is reduced and restart the solution procedure at the
C.......the current time.

        ITIMR=ITIMR + 1
        DO 680 I=1,NODES
C....for picard iteration
        PIPSI(I)=PTPSI(I)
C....for newton iteration
        PSIH (I)=PTPSI(I)
  680   CONTINUE
        GO TO 99
      ENDIF

C.....If the solution needs more iteration,
C     Improve the guessed solution, and iterate again

 340  CONTINUE
      EPSLON2 = EPSLON1
      EPSLON1 = EPSLON

        if(newton.eq.1) then
C......improve PSIH value for next iteration
C
C--------------Picard Iteration
C
           DO 350 I=1,NODES
              PSIH(I)=0.5*(PTPSI(I)+PSI(I))
              PIPSI(I) = PSI(I)
 350       CONTINUE
        else
c
c---------------Newton Raphson Iteration
c
c save the pressures at previous iteration, psih(i) to pipsi(i)
c and add the pressure increment tp pipsi(i) to obtain the pressure,
C psih(i), at the end of this iteration.
c--------------------------------------------------------------------
c
            do 353 i=1,nodes
            pipsi(i)=psih(i)
            psih(i) = psi(i) + pipsi(i)
353         continue 
        endif
c            
c ..... go back to the iteration loop
c
            go to 900
        
C====================================================================
C
C           END OF THE ITERATION LOOP
C
C====================================================================

500   CONTINUE

C.....If the solution converges to the specified tolerance, print the
C     results and march to the next time step.
c
c.....for newton scheme
c     add the pressure increment to the pressure values at the previous
c     iteration to obtain the new pressure head values
c
        if(newton.NE.1) then
           do 502 i=1,nodes
           psi(i) = psi(i) + psih(i)
502        continue
        endif

      IF(ISTATE.NE.1)THEN
c.......update the time to the current time
        TIME = TIME+DELT
        WRITE(*,1121) ITIMR,TIME,LA,iter,DELT
1121    FORMAT(' ',1x,'# DT RDT=',I2,' TIME = ',
     %  E10.2,' TIME STP ',I5,' no. iter.=',i4,' DT =',E13.5)
      
      ELSE
        IF(ISTATE.EQ.1) WRITE(11,1122) iter
1122    FORMAT(' ',/,
     *  10('='),'STEADY-STATE RESULTS', 10('='),/,
     *  10('-'),' NO. OF ITERATION = ',I4,10('-'))
      ENDIF
      
C.....Calculate the mass balance

      CALL MATBAL(PTPSI,F,Q,NBC,ISTATE,KODE,INTEG,
     &CQIN,CQOUT,CQNET,CCIWC,CERRWC)

C.....Check if the time is the desired print-out time.

      IF(TIME.GE.TIMOUT(IP)) THEN
         WRITE(11,1120)  TIME
         write(20,'(a20)') 'zone'
 1120    FORMAT (' ',/,50('-'),//,' SIMULATION RESULT AT TIME = ',
     &   E13.5,//,6X,'Z',13X,'PSI',6X,'THETA',/)
         Z=DELZ*FLOAT(NELEM)
         DO 605 I=1,NODES
         M = KODE(I)
         IF(I.EQ.NODES) M=KODE(NELEM)
         S = SE(PSI(I),M)
         WC = S*(WCS(M) - WCR(M))+WCR(M)
         WRITE(11,1011) Z,PSI(I),WC
         WRITE(20,1011) Z,PSI(I),WC
1011     FORMAT(3E13.5)
         Z=Z-DELZ
 605     CONTINUE
         IP = IP + 1

C..Prints the mass balance for this output time step

         CALL MBALPR(CQIN,CQOUT,CQNET,CCIWC,CERRWC)
      ENDIF

      IF(ISTATE.EQ.1) GO TO 710

C...Determine time step size for next time step

      IF(NFLAG.EQ.1) DELT = DELTOLD
      DELT1=DELT
      IF(ITER.LE.MAXIT) DELT=XMULT*DELT
      IF(DELT.GT.DELMAX) DELT=DELMAX
      
      IF(TIME+DELT.GE.TIMOUT(IP)) THEN
          DELTOLD = DELT
          NFLAG = 1
          DELT=TIMOUT(IP)-TIME
      ELSE
          NFLAG = 0
      ENDIF

C......Store PSI values for the next time step

      DO 700 I=1,NODES
      TPSI(I)=  PTPSI(I)
      PTPSI(I)= PSI(I)
      PIPSI(I)= PSI(I)
      PSIH(I)=  PSI(I)
700   CONTINUE

C.......Check if it is a steady-state flow problem or
C       simulation time exceeds the one specified

710   IF(ip.gt.iprdt) GO TO 711
       
c.......Check if the solution converges during the time step
C        if it converges, jflag. ne.1, advance to a new time
c
      IF(JFLAG.ne.1) go to 98
C
C if the solution does not converge, dump solutions at the last
C time step to RESTAR. DAT file.
C
      WRITE(*,'(A/)')' A restart file has been generated.'
      WRITE(30,'(E13.5)') TIME
      WRITE(30,'(5E13.5)') CQIN,CQOUT,CQNET,CCIWC,CERRWC
      DO 705 I=1,NODES
      WRITE(30,1012) PTPSI(I),TPSI(I)
1012  FORMAT(2E13.5)
705   CONTINUE

711   CONTINUE
      CALL GETTIM(HH,MM,SEC,HD)
      TIME1 =3600.*HH+60.*MM+SEC+0.01*HD
      CPU = TIME1 - TIME0
      WRITE(11,1014) CPU
      WRITE(*,1014) CPU
1014  FORMAT('    CPU TIME = ',E13.5,' SECONDS')
      STOP
      END

C--------------------------------------------------------------
      SUBROUTINE TRIDIA(NN)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON A(501),B(501),C(501),D(501),ANS(501)
      DIMENSION BB(501),GG(501)

      BB(1)=B(1)
      GG(1)=D(1)/B(1)
      DO 10 I=2,NN
      I1=I-1
      BB(I)=B(I)-A(I)*C(I1)/BB(I1)
      GG(I)=(D(I)-A(I)*GG(I1))/BB(I)
 10   CONTINUE

C.....PERFORM BACK SUBST.

      ANS(NN)=GG(NN)
      N1=NN-1
      DO 15 J=1,N1
      I=NN-J
      ANS(I)=GG(I)-C(I)*ANS(I+1)/BB(I)
 15   CONTINUE
      RETURN
      END

      SUBROUTINE MATBAL (PTPSI,F,Q,NBC,ISTATE,KODE,INTEG,
     *CQIN,CQOUT,CQNET,CCIWC,CERRWC)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION PTPSI(501),F(2,501),Q(2),NBC(2),KODE(501)
      COMMON A(501),B(501),C(501),D(501),PSI(501)
      COMMON/BM/DELT,DELZ,EPS,NODES
      COMMON/SL/XKS(10),ALFA(10),XN(10),WCS(10),WCR(10),SS(10),IDM(10)

      QIN  = 0.0
      QOUT = 0.0
      CDWC = 0.0
      GRAV = 0.0
      IF (INTEG.EQ.2) GRAV = DELZ
      IF(ISTATE.EQ.1) DELT = 1.0

C.....Computes water inflow/outflow thru upper boundary

      IF (NBC(1).EQ.1) GO TO 1
      Q(1)=-((PSI(1)-PSI(2)+GRAV)*EPS*F(1,1)/DELZ
     1+(PTPSI(1)-PTPSI(2)+GRAV)*(1.-EPS)*F(1,1)/DELZ)
1     CONTINUE

C.....Q positive is inflow to the system

      IF(Q(1).LT.0.0)THEN
          QOUT=QOUT-Q(1)*DELT
      ELSE
          QIN=QIN+Q(1)*DELT
      ENDIF

C.....Computes water inflow/outflow thru lower boundary

      IF (NBC(2).EQ.1) GOTO 2
      Q(2)=(PSI(NODES-1)-PSI(NODES)+GRAV)*EPS*F(2,NODES-1)/DELZ
     1+(PTPSI(NODES-1)-PTPSI(NODES)+GRAV)*(1-EPS)*F(2,NODES-1)/DELZ
 2    CONTINUE

C.....If Q is positive, it is outflow

      IF(Q(2).LT.0.0)THEN
      QIN=QIN-Q(2)*DELT
      ELSE
      QOUT=QOUT+Q(2)*DELT
      ENDIF

C.....Computes change in water storage

      CDWC = 0.0
      NELEM=NODES-1
      DO 10 K=1,NELEM
      M=KODE(K)
      K1 = K
      K2 = K + 1
      WCE = (WCS(M) - WCR(M))
C
C.....Moisture content at previous time
C
      IF(PTPSI(K1).LT.0.0) THEN
        PWC1 = SE(PTPSI(K1),M)*WCE+WCR(M)
      ELSE
        PWC1 = SS(M)*PTPSI(K1) + WCS(M)
      ENDIF

      IF(PTPSI(K2).LT.0.0) THEN
        PWC2 = SE(PTPSI(K2),M)*WCE+WCR(M)
      ELSE
        PWC2 = SS(M)*PTPSI(K2) + WCS(M)
      ENDIF

      PWC  = (PWC1 + PWC2)*DELZ*0.5
C
C.....Moisture content at current time
C
      IF(PSI(K1).LT.0.0) THEN
        WC1 = SE(PSI(K1),M)*WCE+WCR(M)
      ELSE
        WC1 = SS(M)*PSI(K1) + WCS(M)
      ENDIF
      IF(PSI(K2).LT.0.0) THEN
        WC2 = SE(PSI(K2),M)*WCE+WCR(M)
      ELSE
        WC2 = SS(M)*PSI(K2) + WCS(M)
      ENDIF

      WC  = (WC1 + WC2)*DELZ*0.5

C.....Change in moisture content in the element during delt

      DWC = WC - PWC

C.....CDWC sum of changes in water content in all elements

      CDWC = CDWC + DWC
10    CONTINUE
C
C.....Net discharge during this time step
C
      QNET = QIN - QOUT
C
C.....Errors in mass balance during this time step
C
      ERRWC = QNET - CDWC

C.....Cumulative error over simulation periods

      CQIN  = CQIN  + QIN
      CQOUT = CQOUT + QOUT
      CQNET = CQNET + QNET
      CCIWC = CCIWC + CDWC
      CERRWC = CERRWC + ERRWC
      RETURN
      END


      SUBROUTINE MBALPR(CQIN,CQOUT,CQNET,CCIWC,CERRWC)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	percerr=0.0
      If (cqnet.ne.0.) PERCERR = (CERRWC/CQNET)*100.
      WRITE(11,475)
      WRITE(11,471)
      WRITE(11,473) CQIN,CQOUT,CQNET,CCIWC,CERRWC,PERCERR
      WRITE(11,475)
 471  FORMAT(/11X,'* * * * MATERIAL BALANCE ERROR ANALYSIS * * * * ')
 473  FORMAT(/,5X,'CULM. WATER IN        = ',E11.4,
     1       /,5X,'CULM. WATER OUT       = ',E11.4,
     2       /,5X,'CULM. WATER IN & OUT  = ',E11.4,
     3       /,5X,'CULM. CHANGE IN STRG. = ',E11.4,
     4       /,5X,'CULM. WATER ERROR     = ',E11.4,
     5       /,5X,'ERROR IN % (ERROR/CQNET*100) = ',E11.4//)
 475  FORMAT (26('===='))
      RETURN
      END
C
C---- FUNCTION TO EVALUATE DIMENSIONLESS SATURATION
      FUNCTION SE(H,M)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/SL/XKS(10),ALFA(10),XN(10),WCS(10),WCR(10),SS(10),IDM(10)

      SE=1.0
      IF(H.GE.0.0) RETURN
      IF(IDM(M).EQ.1)THEN
         SE=(1.0+DABS(ALFA(M)*H)**XN(M))**(-(1.0-1.0/XN(M)))
       ELSE IF(IDM(M).EQ.2)THEN
         SE=DEXP(ALFA(M)*H)
       ELSE IF(IDM(M).EQ.3)THEN
         EE=0.5*ALFA(M)*H
         SE=DEXP(EE)*(1.0+DABS(EE))**(2.0/(XN(M)+2.0))
       ELSE IF(IDM(M).EQ.4)THEN
         SE=1.0/(1.0+(ALFA(M)*H)**2.0)
      ENDIF
      RETURN
      END

C---- FUNCTION TO SPECIFIC MOISTURE CAPACITY
      FUNCTION CM(H,M)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/SL/XKS(10),ALFA(10),XN(10),WCS(10),WCR(10),SS(10),IDM(10)

      CM=SS(M)
      IF(H.GE.0.) RETURN
      IF(IDM(M).EQ.1)THEN
         XM=1.0-1.0/XN(M)
         S=(1.0+DABS(ALFA(M)*H)**XN(M))**(-XM)
         GAMMA1=1.0/XM
         SEM = (S**GAMMA1)
         CM=ALFA(M)*(XN(M)-1.0)*(WCS(M)-WCR(M))*SEM*
     *      (1.0-SEM)**XM
       ELSE IF(IDM(M).EQ.2)THEN
         CM=(WCS(M)-WCR(M))*ALFA(M)*DEXP(ALFA(M)*H)
       ELSE IF(IDM(M).EQ.3)THEN
         HA=0.5*ALFA(M)
         CM=SE(H,M)*HA*(WCS(M)-WCR(M))*(2.0/((XN(M)+2.0)*
     *      (1.0+HA*H))-1.0)
       ELSE IF(IDM(M).EQ.4)THEN
         CM=-2.0*H*(WCS(M)-WCR(M))*(ALFA(M)*SE(H,M))**2.0
      ENDIF
      RETURN
      END

C   FUNCTION TO EVLAUTE RELATIVE CONDUCTIVITY
      FUNCTION XK(H,M)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/SL/XKS(10),ALFA(10),XN(10),WCS(10),WCR(10),SS(10),IDM(10)

      XK=XKS(M)
      IF(H.GE.0.) RETURN
      IF(IDM(M).EQ.1)THEN
         GAMMAK=1.0-1.0/XN(M)
         GAMMA1=1.0/GAMMAK
         S=(1.0+DABS(ALFA(M)*H)**XN(M))**(-GAMMAK)
         XK=XKS(M)*DSQRT(S)*(1.0-(1.0-S**(GAMMA1))**GAMMAK)**2
       ELSE IF(IDM(M).EQ.2. OR .IDM(M).EQ.3)THEN
         XK=XKS(M)*DEXP(ALFA(M)*H)
       ELSE IF(IDM(M).EQ.4)THEN
         XK=XKS(M)*SE(H,M)**4.0
      ENDIF
      RETURN
      END

C
C
      FUNCTION DK(H,PH,M)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/SL/XKS(10),ALFA(10),XN(10),WCS(10),WCR(10),SS(10),idm(10)
      DK = 0.0
      IF(H.EQ.PH) RETURN
      xkh = xk(h,m)
      xkp = xk(ph,m)
      dk =(xkh-xkp)/(h-ph)
      return
      RETURN
      END
C
C
      FUNCTION DC(H,PH,M)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      COMMON/SL/XKS(10),ALFA(10),XN(10),WCS(10),WCR(10),SS(10),idm(10)
      DC = 0.0
      IF(H.EQ.PH) RETURN
      dch = cm(h,m)
      dcp = cm(ph,m)
      dc =(dch-dcp)/(h-ph)
      return
      END



